#ifndef _AUDIO_C_
#define _AUDIO_C_

#include <gst/gst.h>
#include <glib.h>

#include "audio.h"

#include "functionality.h"

/* TODO:
	- cbBus() is incomplete and probably has a bug
	- set_element_state() should be print information aobut
		the element if the state change fails
	- the filepath (and the file) for recording should be
		handled in a more approprite way (as parameter for
		audio_initialize?)
	- Element's state should(?) checked before
		changing it?
	- Automatic slide of end-of-stream message is a really
	  ugly hack at the moment. Make it beautiful!

	BUG: after awhile of recording "no space left on the resource"
		 error happends (verify that bug certainly lies in the rec)
*/


/* ==== preprocessor macros ==== */
/* Defines volume [0.0, 10.0] for Gstreamer pipelines */
#define VOLUME (7.0f)
#define VOLUME2 (10.0f)
typedef enum {
	NONE = 0x0,
	RECORDING,
 	LISTENING
} State;

State system_state;

/* ==== Helper functions ==== */
GstElement *create_generic_playback_pipeline( gchar *name, gfloat volume );
GstElement *create_recording_pipeline( gchar *name, gchar *target );
void destroy_pipeline( GstElement *pipeline );

void set_element_state( GstElement *element, GstState state );
void set_uri( GstElement *element, gchar *name );

gboolean cbBus( GstBus *element, GstMessage *msg, gpointer data );

/* ==== Globals ==== */
GstElement *player = NULL;
GstElement *recorder = NULL;

void audio_initialize()
{
	g_assert( player == NULL );
	g_assert( recorder == NULL );

	#if DEBUG
	g_print("Audio initialize:\n");
	#endif

	system_state = NONE;

	player = create_generic_playback_pipeline( "player", VOLUME );
	if( player == NULL ) {
		g_warning("Creating generic player pipeline failed\n");
	}

	// FIXME
	recorder = create_recording_pipeline( "recording", "/tmp/mobikid_rec.wav" );
	if( recorder == NULL ) {
		g_warning("Creating recording pipeline failed\n");
	}

	#if DEBUG
	g_print("Initializing audio done\n");
	#endif
}

void audio_deinitialize()
{
	/* Because pipelines (eg. player) might (and will be)
	   NULL during the natural execution of the program,
	   any asserts for them aren't needed here
	 */

	#if DEBUG
	g_print( "Audio_deintialize()\n" );
	#endif

	if( player != NULL ) {
		destroy_pipeline( player );
	}

	if( recorder != NULL ) {
		destroy_pipeline( recorder  );
	}

	system_state = NONE;

	#if DEBUG
	g_print("Deinitializing audio done\n");
	#endif
}


void audio_recording_state_start( gchar *song )
{

	g_assert( player != NULL );
	g_assert( recorder != NULL );



	#if DEBUG
	g_print("play and record: start\n");
	#endif

	// at this point player and recorder should be set
	// NULL state ?
	destroy_pipeline( player );
	player = create_generic_playback_pipeline( "player", VOLUME );





	set_uri( player, song );

	/*-------------------------------------------*
	GstClock *gstclock;
	gstclock = gst_pipeline_get_clock(player);
	gst_pipeline_use_clock (recorder, gstclock);
	/*-------------------------------------------*/



	set_element_state( recorder, GST_STATE_PLAYING );
	set_element_state( player, GST_STATE_PLAYING );

	system_state = RECORDING;

	#if DEBUG
	g_print("play and record: stop\n");
	#endif
}

void audio_recording_state_stop()
{
	g_assert( player != NULL );
	g_assert( recorder != NULL );

	#if DEBUG
	g_print("stop_play and record: stopping...\n");
	#endif

	set_element_state( player, GST_STATE_NULL );
	set_element_state( recorder, GST_STATE_NULL );

	system_state = RECORDING;

	#if DEBUG
	g_print("stop_play_and_record: stopped\n");
	#endif
}


/* Temporary fix to get the song playing properly
	-> looses the synchronizatoin
*/
GstElement *tmp = NULL;
void audio_listening_state_start( gchar *song, gchar *record  )
{
	g_assert( player != NULL );

	#if DEBUG
	g_print("play and listen: start\n");
	#endif

	g_print("\t\tsong is '%s'\n", song);

	destroy_pipeline( player );
	player = create_generic_playback_pipeline( "player", VOLUME );
	set_uri( player, song );

	tmp = create_generic_playback_pipeline( "temp", VOLUME2 );
	set_uri( tmp, record );

	/*-------------------------------------------*
	GstClock *gstclock;
	gstclock = gst_pipeline_get_clock(player);
	gst_pipeline_use_clock (tmp, gstclock);
	/*-------------------------------------------*/



	set_element_state( tmp, GST_STATE_PLAYING );
	set_element_state( player, GST_STATE_PLAYING );

	system_state = LISTENING;

	#if DEBUG
	g_print("play and listen: stop\n");
	#endif
}

void audio_listening_state_stop()
{
	g_assert( player != NULL );

	#if DEBUG
	g_print("stop_play and listen: stopping...\n");
	#endif

	set_element_state(tmp, GST_STATE_NULL );

	set_element_state(player, GST_STATE_NULL );

	system_state = LISTENING;

	#if DEBUG
	g_print("stop_play_and_listen: stopped\n");
	#endif
}

/* Really dirty and really quick hack to commenting work
   Needs to be totally rewritten FIXME
*/
#include <glib.h>

void audio_trigger_comment()
{
	GstElement *comment = NULL;

	{
	comment = gst_element_factory_make( "playbin", "comment" );
	if( comment == NULL ) {
		g_warning("[%s] Couldn't create pipeline\n", "comment" );
	}

	g_object_set( G_OBJECT(comment), "volume", VOLUME, NULL );
	}

//	system_state = NONE;
	g_print("Here comes the comment!\n");

	comment = create_generic_playback_pipeline( "comment", VOLUME );
	g_assert( comment != NULL );
	set_element_state( comment, GST_STATE_NULL );

	set_uri(comment, "file:///usr/share/mobikid/sounds/mobikid_loistavaa.mp3");


	set_element_state( comment, GST_STATE_PLAYING );

	g_usleep( 2 * G_USEC_PER_SEC );

	set_element_state( comment, GST_STATE_NULL );

	/* FIXME */
//	from_recording_to_listening();

	/* change element to null here */
}

/*
=================================

Helper functions

=================================
*/

GstElement *create_generic_playback_pipeline( gchar *name, gfloat volume )
{
	GstElement *templine = NULL;
	GstBus *bus = NULL;

	templine = gst_element_factory_make( "playbin", name );
	if( templine == NULL ) {
		g_warning("[%s] Couldn't create pipeline\n", name );
		return NULL;
	}

	bus = gst_pipeline_get_bus( GST_PIPELINE(templine) );
	if( bus == NULL ) {
		g_warning("[%s] Couldn't get bus from pipeline\n", name );
		return NULL;
	}

	gst_bus_add_watch( bus, cbBus, NULL );
	g_object_unref( bus );

	set_element_state( templine, GST_STATE_NULL );

	g_object_set( G_OBJECT(templine), "volume", volume, NULL );

	return templine;
}

GstElement *create_recording_pipeline( gchar *name, gchar *target )
{
	GstElement *source = NULL,
			   *encoder = NULL,
			   *sink = NULL,
			   *templine = NULL;

	templine = gst_pipeline_new( name );
	if( templine == NULL ) {
		g_warning("Templine creation failed\n");
	}
	/* HACK to disable the double slide effect of two eos messages,
		this hides a bug (the recording stops complaining out of space
		error)
	gst_bus_add_watch( gst_pipeline_get_bus( GST_PIPELINE(templine)),
						cbBus, NULL );
	*/

	source = gst_element_factory_make( "dsppcmsrc", "source" );
	if( source == NULL ) {
		g_warning("Audio (rec): Source is null\n");
	}

	encoder = gst_element_factory_make( "wavenc", "encoder" );
	if( encoder == NULL ) {
		g_warning("Audio (rec): Encoder is null\n");
	}

	sink = gst_element_factory_make( "filesink",  "sink" );
	if( sink == NULL ) {
		g_warning("Audio (rec): Sink is null\n");
	}
	g_object_set( G_OBJECT(sink), "location",
				  target, NULL );

	gst_bin_add_many( GST_BIN(templine), source, encoder, sink, NULL );
	if( gst_element_link_many( source, encoder, sink, NULL ) == FALSE )
	{
		g_warning("Audio (rec): Linking elements FAILED\n");
	}

	set_element_state( templine, GST_STATE_NULL );

	return templine;
}

void destroy_pipeline( GstElement *pipeline )
{
	g_assert( pipeline != NULL );

	#if DEBUG
	g_print("Destroying pipeline\n");
	#endif

	set_element_state( pipeline, GST_STATE_NULL );
	g_object_unref( pipeline );

	pipeline = NULL;
}

void set_uri( GstElement *element, gchar *uri )
{
	#if DEBUG
	gboolean valid = FALSE;

	valid = gst_uri_is_valid( uri );
	if( valid == FALSE ) {
		g_warning("Given URI '%s' isn't valid\n", uri );
	}

	g_print("URI to set is '%s'\n", uri );
	#endif

	g_object_set( G_OBJECT(element), "uri", uri, NULL );
}

void set_element_state( GstElement *element, GstState state )
{
	g_assert( element != NULL );

	GstStateChangeReturn ret = GST_STATE_CHANGE_FAILURE;

	#if DEBUG
	g_print("Setting element's state\n");
	#endif

	ret = gst_element_set_state( element, state );
	if( ret == GST_STATE_CHANGE_FAILURE ) {
		g_warning("Not able to change element's state\n");
		/* FIXME print info about element and state? ?*/
	}
}

gboolean cbBus( GstBus *bus, GstMessage *msg, gpointer data )
{
	/*
	http://gstreamer.freedesktop.org/data/doc/gstreamer/head/gst-plugins-base-plugins/html/gst-plugins-base-plugins-playbin.html

	After error or end-of-stream playbin should be put
	to READY or NULL state.

	IDEA: give playbin/pipeline as argument and
			then just set it to null
	*/

	/* notice the hack at create_recording_pipeline() function */
	switch( GST_MESSAGE_TYPE( msg ) ) {
		case GST_MESSAGE_EOS: { /* FIXME might be a bug here */

		/* really ugly hack solution */
		#if 1
		switch( system_state ) {
		case RECORDING :
			g_print("switch| End of recording\n");
			//audio_trigger_comment();
			from_recording_to_listening();
		break;
		case LISTENING :
			g_print("switch| End of listening\n");
			from_listening_to_sending();
		break;
		case NONE :
			g_print("switch| system state is NONE");
		break;
		default:
			g_warning("switch| Awful error\n");
		break;
		}
		#endif
		g_print("End of stream\n");

		}
		break;
		case GST_MESSAGE_ERROR: {
		gchar *debug;
		GError *err;

		gst_message_parse_error( msg, &err, &debug );
		g_free( debug );

		g_print( "Error: %s\n", err->message );
		g_error_free (err);
		break;
		}
		#if 0
		case GST_MESSAGE_CLOCK_PROVIDE : // 512
			g_print("Clock provider message\n");
		break;
		case GST_MESSAGE_STATE_CHANGED : { // 64
			GstState old = 0, new = 0, pending = 0;

			g_print("State changed message\n");
			//gst_message_parse_state_changed(  msg, &old, &new, &pending );
			//g_print( "\t\told %u, new %u, pending %u\n");
		break;
		}
		case GST_MESSAGE_TAG:
			g_print("Tag message\n");
		break;
		case GST_MESSAGE_NEW_CLOCK :
			g_print("gst message new clock\n");
		break;
		case GST_MESSAGE_ASYNC_START :
			g_print("MEssage ASYNC start\n");
		break;
		case GST_MESSAGE_ASYNC_DONE :
			g_print("Message ASYNC done\n");
		break;
		#endif
		#if 0
		default: {
			g_print("Message type: %u\n", GST_MESSAGE_TYPE( msg ) );
		}
		#else
		default:
		#endif
		break;
	}

	return TRUE;
}

#endif

