#ifndef _FUNCTIONALITY_C_
#define _FUNCTIONALITY_C_

#include <glib.h>
#include <gtk/gtk.h>

#include "functionality.h"

#include "audio.h"
#include "video.h"
#include "email.h"
#include "log.h"
#include "ui/ui.h"
#include "ui/songselection.h"
#include "ui/recording.h"

#define REC_URI "file:///tmp/mobikid_rec.wav"
#define REC_FILE "/tmp/mobikid_rec.wav"

gchar *selected_song = NULL;

void from_splash_screen_to_selection() 
{
	slide( RIGHT );
	video_playback_stop();
	video_cleanup();
}

/**
 * @param name The name of the selected song (includes file suffix)
 */
void from_selection_to_recording( gchar *song_name, gchar *pic_name )
{
	selected_song = song_name;
	recording_add_songpicture( pic_name );	
	songselection_add_kidpicture( selected_song );	

	g_message("Song %s selected", song_name);

	#if DEBUG
	g_print("Song's name: %s\n", song_name );
	#endif

	slide( RIGHT );

	audio_recording_state_start( selected_song );	
}

void from_recording_to_selection()
{
	audio_recording_state_stop();

	selected_song = NULL;

	slide( LEFT );	
}

void from_recording_to_listening() 
{
	/* order of these calls is intended,
	 * it reduces the delay before musics starts
	 * in the listening window 
	 */

	audio_recording_state_stop();
	audio_trigger_comment();

	audio_listening_state_start( selected_song, REC_URI );

	slide( RIGHT );
}

void from_listening_to_recording() 
{
	audio_listening_state_stop();

	slide( LEFT );

	audio_recording_state_start( selected_song );
}

void from_listening_to_sending() 
{
	audio_listening_state_stop();

	slide( RIGHT );	
}

void from_sending_to_selection()
{
	slide( RIGHT );
}

void slide( Direction dir )
{
	ui_slide( dir );
}

void quit()
{
	#if DEBUG
		g_print("quit program\n");
	#endif
	g_message( "quit program\n");
	//clean up memory
	gtk_main_quit();
}

void send_email( gchar *address )
{
	#if DEBUG
	g_print("send_email called");
	#endif
	//gchar* email_address=address;
	g_print("send_email()| %s\n", address );	
	//printf("\nEmail to: %s",email_address);
	email_send(address);
}

#endif
