#include <hildon/hildon-program.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <libosso.h>
#include <gst/gst.h>

#include "ui/ui.h"
#include "audio.h"
#include "video.h"
#include "email.h"
#include "log.h"


/*
	According to Coding style -howto headers should have these lines

	#if __cplusplus
	extern "C" {
	#endif

	#if __cplusplus
	}
	#endif

	This will make sure that the code will  work
	when included to c++ project (eg. no name mangling)
*/

#define APPLICATION_NAME "mobikid"
#define APPLICATION_VERSION "0.1"

#define DBUS_APP_NAME "org.maemo.mobikid"

typedef struct
{
	HildonProgram *program;
	HildonWindow *window;
	osso_context_t *ossoContext;
} Application;

gint main( gint argc, gchar *argv[] )
{
	#ifdef DEBUG
		g_print("\n\n\tDEBUG mode is on!\n");
	#else
		g_print("\n\n\tDEBUG mode is off!\n");
	#endif

	//start to log all g_messages and g_warnings
	g_log_set_handler( NULL, G_LOG_LEVEL_MESSAGE | G_LOG_LEVEL_WARNING,
					   log_to_file, NULL);
	log_start();

	g_message( "start program" );

	Application app = { NULL, NULL, NULL };

	gtk_init( &argc, &argv );
	gst_init( &argc, &argv );

	app.ossoContext = osso_initialize( DBUS_APP_NAME,
						APPLICATION_VERSION,
			   			   	TRUE, NULL );
	if( app.ossoContext == NULL ) {
		g_warning("Osso: initialize error\n");
		return OSSO_ERROR;
	}

	app.program = HILDON_PROGRAM( hildon_program_get_instance() );
	g_set_application_name( APPLICATION_NAME );
	hildon_program_set_can_hibernate( app.program, FALSE );

	app.window = ui_initialize();
	if( app.window == NULL ) {
		g_warning("Not able to create the user interface\n");
		return EXIT_FAILURE;
	}

	hildon_program_add_window( app.program, app.window );
	ui_show( app.window );

/*	email_initialize( app.ossoContext );	*/

	audio_initialize();

	gtk_main();

	audio_deinitialize();
	ui_deinitialize();
	osso_deinitialize( app.ossoContext );

	return EXIT_SUCCESS;
}


