#ifndef _CALLBACKS_C_
#define _CALLBACKS_C_

#include <hildon/hildon-program.h> /* for hildon hard key macros */

#include "callbacks.h"
#include "ui.h"
#include "../functionality.h"

/*
	@ret TRUE if event was processed, otherwise FALSE
*/
gboolean cb_hardkey_press( GtkWidget *widget, GdkEventKey *event,
						 gpointer ptr )
{
	/* According to coding style -howto GDK_* events shouldn't be used,
	    instead use HILDON_HARDKEY macros.
	*/
	g_assert( widget != NULL );
	g_assert( event != NULL );

	switch( event->keyval ) {
	case HILDON_HARDKEY_UP :
		return FALSE;
	case HILDON_HARDKEY_RIGHT :
		return FALSE;
	case HILDON_HARDKEY_DOWN :
		return FALSE;
	case HILDON_HARDKEY_LEFT :
		return FALSE;
	case HILDON_HARDKEY_FULLSCREEN :
		return FALSE;
	case HILDON_HARDKEY_ESC :
		quit();
		return TRUE;
	default:
		#if DEBUG
		g_warning("Reached default of switch in cbHardKeyPress(---)\n");
		#endif
		break;
	}

	return FALSE;
}
/* Not really used -> might help to prevent event handling
	while sliding -> no accidental slides over multiple views
	at once */
gboolean cb_mousebutton_press( GtkWidget *widget, GdkEventKey *event,
							   gpointer ptr )
{
	g_assert( widget != NULL );
	g_assert( event != NULL );

	#if DEBUG
	g_print("\n\n\tMouseButton press\n");
	#endif

	return FALSE;
}

/*
 * Makes the screen slide. Handy for testing
 * but shouldn't be used for actual application
 * view changes
 */
gboolean cb_slide( GtkWidget *widget, gpointer data )
{
	g_assert( widget != NULL );
	g_assert( ((Direction) data) == LEFT ||
			  ((Direction) data) == RIGHT );

	ui_slide( (Direction) data );

	return TRUE;
}

gboolean cb_quit( GtkWidget *widget, gpointer data )
{
	g_assert( widget );

	quit();

	return TRUE;
}

// ===============================
// splash screen to selection
gboolean cb_from_splash_screen_to_selection( GtkWidget *widget, GdkEvent *event,
											 gchar *ptr )
{
	g_assert( widget != NULL );
	g_assert( event != NULL );

	from_splash_screen_to_selection();

	return TRUE;
}

// ===============================
// Selection to recording
/**
 * @param song_pic_filenames
 *         	1. element: Name of the selected song (includes type eg. .mp3 )
 *			2. element: Name the picture of the selected song (includes type eg. .gif )
 */
gboolean cb_from_selection_to_recording( GtkWidget *widget, GdkEvent *event,
										 GSList *song_pic_filenames )
{
	gchar *song_name = NULL, *pic_name = NULL;
	g_assert( widget != NULL );
	g_assert( song_pic_filenames != NULL );

	song_name = (gchar *) g_slist_nth_data( song_pic_filenames, 0);
	pic_name = (gchar *) g_slist_nth_data( song_pic_filenames, 1);
	g_print("\nsong name is %s\n", song_name);
	g_print("\npicture name is %s\n", pic_name);
	from_selection_to_recording( song_name, pic_name );

	return TRUE;
}

// ===============================
// Recording -> Selection
gboolean cb_from_recording_to_selection( GtkWidget *widget, GdkEvent *event,
										 gchar *name )
{
	g_assert( widget != NULL );
	g_assert( name != NULL );

	from_recording_to_selection();

	return TRUE;
}

// ===================================
// Recording -> Listening
gboolean cb_from_recording_to_listening( GtkWidget *widget, GdkEvent *event,
										 gchar *name )
{
	g_assert( widget != NULL );
	g_assert( name != NULL );

	from_recording_to_listening();

	return TRUE;
}

// ================================
// Listening to Recording
gboolean cb_from_listening_to_recording( GtkWidget *widget, GdkEvent *event,
										 gchar *name )
{
	g_assert( widget != NULL );
	g_assert( name != NULL );

	from_listening_to_recording();

	return TRUE;
}

// ================================
// Listening to Sending
gboolean cb_from_listening_to_sending( GtkWidget *widget, GdkEvent *event,
									   gchar *name )
{
	g_assert( widget != NULL );
	g_assert( name != NULL );

	from_listening_to_sending();

	return TRUE;
}

gboolean cb_from_sending_to_selection( GtkWidget *widget, GdkEvent *event,
									   gchar *name )
{
	g_assert( widget != NULL );
	g_assert( name != NULL );

	from_sending_to_selection();

	return TRUE;
}

gboolean cb_send_email( GtkWidget *widget, GdkEvent *event,
						gchar *name )
{
	g_assert( widget != NULL );
	g_assert( event != NULL );

	send_email( name );

	return TRUE;
}

#endif
