#ifndef _SONGSELECTION_C_
#define _SONGSELECTION_C_

#include "callbacks.h"
#include <string.h> /* for strlen() and pals */
#include <glib.h> /* Double-ended queue */

/* Directories */
#define PICTURE_DIR "/usr/share/mobikid/pictures/"
#define SONG_DIR "/usr/share/mobikid/songs/"

/* Images */
#define BIG_FRAME "/usr/share/mobikid/pics/song_frame.png"
#define KID_FRAME "/usr/share/mobikid/pics/kid_frame.png"
#define KID_PICTURE "/usr/share/mobikid/pics/kid.png"
#define DEFAULT_PICTURE "/usr/share/mobikid/pics/nopic.png"

/* Backgrounds */
#define SONG_SELECTION "/usr/share/mobikid/pics/puzzle1.png"

const gchar *standard_pic = "file:///usr/share/mobikid/pics/nopic.png"; 
const guint max_rows = 3, max_columns = 4;

/* It's a list of song names with their fullpaths,
	used in comparsion when inserting kid pictures
*/
GSList *song_list = NULL; 
GSList *picture_list = NULL;
GtkWidget *fixed = NULL;

void add_button( gchar *song_buffer, gchar *pic_buffer, int current_row, int current_column);
gint string_compare( gconstpointer a, gconstpointer b );
GSList *scan_and_sort_directory( gchar *directory );


void songselection_destroy()
{
	/* FIXME bug, destroys the list but not the allocated memory
	   blocks that are pointed by list elements */
	gint counter = 0;
	for( counter = 0; counter < g_slist_length (song_list); counter++ )
	{
		g_free(g_slist_nth_data(song_list, counter));  
	} 
	g_slist_free( song_list ); 
	for( counter = 0; counter < g_slist_length (picture_list); counter++ )
	{
		g_free(g_slist_nth_data(picture_list, counter));  
	} 
	g_slist_free( picture_list );
	song_list = NULL;
	picture_list = NULL; 
	fixed = NULL;
}

GtkWidget *songselection_create()
{
	g_assert( fixed == NULL );

	GtkWidget *background = NULL;
	gchar *pic_buffer = NULL;
	gchar *song_buffer = NULL;

	fixed = gtk_fixed_new();

	background = gtk_image_new_from_file( SONG_SELECTION );
	gtk_fixed_put( GTK_FIXED(fixed), background, 0, 0 );

	/* ========================================================== */
	#if DEBUG
	g_print("UI| create song selection screen\n");
	#endif
		
	GSList *pictures = NULL;
	pictures = scan_and_sort_directory( PICTURE_DIR );
	if( pictures == NULL ) {
		g_warning("List of pictures is NULL");
	}

	GSList *songs = scan_and_sort_directory( SONG_DIR );
	if( songs == NULL ) {
		g_warning("List of songs is NULL");
	}

	guint current_row = 0, current_column = 0; // [0,2] [0,5] respectively
	//gboolean running = TRUE;
	gint counter = 0;
	for( counter = 0; counter < g_slist_length (songs); counter++ )
	{
		gchar *picture = NULL, *song = NULL;

		song = g_slist_nth_data( songs, counter );
	
		guint length = 0;
		gint index = 0;

		length = g_slist_length( pictures );
		for( index = 0; index < length; ++index ) {
			gchar *element = (gchar *) g_slist_nth_data( pictures, index );
			//gint len = strlen(element);
			int ret = string_compare( song, element );
			if( ret == 0 ) {
				#if DEBUG
				g_print("%s == %s\n", song, element );
				#endif
				picture = element;
				break;
			}
		}
		/* String are used as helpers to add absolute path for
		   picture and URI for the song */
		GString *pic_string = NULL;
		GString *song_string = NULL;
 
		// if no picture was found for a certain song use default picture 
		if( picture == NULL )
		{
			pic_string = g_string_new( DEFAULT_PICTURE );
		} 
		else
		{
			pic_string = g_string_new( PICTURE_DIR );
			pic_string = g_string_append( pic_string, picture );
		}	
				
		/* FIXME does this leak? */
		song_string = g_string_new( SONG_DIR );
		g_string_prepend( song_string, "file://" );
		
		song_string = g_string_append( song_string, song );

		pic_buffer = g_string_free( pic_string, FALSE );
		song_buffer = g_string_free( song_string, FALSE );

		#if 1
		g_print("pic_buffer '%s'\n", pic_buffer );
		g_print("song_buffer '%s'\n", song_buffer );
		#endif

		song_list = g_slist_append( song_list, song_buffer );
		picture_list = g_slist_append( picture_list, pic_buffer );
		g_print("song_buffer added to song_list list\n");

		if( current_column >= max_columns ) {
			current_column = 0;
			current_row++;
			if( current_row >= max_rows ) {
				#if DEBUG 				
				g_print("out of screen space\n");
				#endif
				break;		
			}
		}

		add_button( song_buffer, pic_buffer, current_row, current_column);
		current_column++;	
		/* FIXME leaks a ton of memory
		Freeing of pic_buffer and song_buffer would free the song names
		and therefore causing a bug (eg. the songs wouldn't play). 
		Still the memory needs to freed. 
		*/

		/*		
		g_free( pic_buffer );
		g_free( song_buffer );
		*/
	}

	g_slist_free( pictures );
	g_slist_free( songs );

	return fixed;
}
/**
adds a picture of a kid to every selected song
a picture and its frame have to be added to the song selection screen as event 
boxes, otherwise those would not be displayed in front of the song picture  
**/
void songselection_add_kidpicture( gchar *song )
{
	g_assert( fixed != NULL );	
	g_assert( song_list != NULL );

	#if DEBUG
	g_print( "\t\tAdd kid picture\n" );		
	#endif

	GtkWidget *frame = NULL; 
	GtkWidget *image = NULL;
    GtkWidget *event_box = NULL, *event_frame = NULL;

	GSList *temp = g_slist_find_custom( song_list, song, string_compare );
	if( temp == NULL ) {	
		g_warning("Song named as '%s' isn't found from kiddies\n", song );
	} else {
		g_print("The song was found from kiddies\n");
	}
	gint position = g_slist_position( song_list, temp );

	// padding between the left/top "border" and the buttons
	const guint padding_left = 124, padding_top = 48;
	// padding between the buttons (in pixels)
	const guint spacing_x = 24, spacing_y = 32;
	// size of song frame
	const guint frame_width = 120, frame_heigth = 120;
	// position of the kid pictures frame (relative to pictures frames position)
	const guint small_frame_position_x = 57;
	const guint small_frame_position_y = 57;
	// kid pictures position (relative to small frame's position
	const guint picture_position_x = 8;
	const guint picture_position_y = 8;

	guint xcoord = 0;
	guint ycoord = 0;

	guint column  = position % max_columns;
	guint row = position / max_columns;

	xcoord += (column * frame_width) + padding_left + small_frame_position_x;
	ycoord += (row * frame_heigth) + padding_top + small_frame_position_y;
		
	xcoord += spacing_x * column;
	ycoord += spacing_y * row;

	frame = gtk_image_new_from_file( KID_FRAME );
	image = gtk_image_new_from_file( KID_PICTURE ); 
	event_box = gtk_event_box_new();
	event_frame = gtk_event_box_new();
 
	gtk_container_add( GTK_CONTAINER(event_box), image );
	gtk_container_add( GTK_CONTAINER(event_frame), frame );
		
	gtk_fixed_put( GTK_FIXED(fixed), event_frame, xcoord, ycoord );

	xcoord += picture_position_x;
	ycoord += picture_position_y;
	gtk_fixed_put( GTK_FIXED(fixed), event_box, xcoord, ycoord );

	gtk_widget_show_all( GTK_WIDGET(fixed) );
}

void add_button( gchar *song_buffer, gchar *pic_buffer, int current_row, int current_column)
{
	GtkWidget *frame = NULL; 

	// padding between the left/top "border" and the buttons
	const guint padding_left = 124, padding_top = 48; // 28??
	// padding between the buttons (in pixels)
	const guint spacing_x = 24, spacing_y = 32;
	// size of song frame
	const guint frame_width = 120, frame_heigth = 120;
	// buttons position (relative to frame's position)
	const guint button_position_x = 9, button_position_y = 9;

	// calculate frame's position
	guint xcoord = 0;
	guint ycoord = 0;

	xcoord += (current_column * frame_width) + padding_left;
	ycoord += (current_row * frame_heigth) + padding_top;

	xcoord += spacing_x * current_column;
	ycoord += spacing_y * current_row;
	
	frame = gtk_image_new_from_file( BIG_FRAME );

	GtkWidget *event_box = gtk_event_box_new();
	GtkWidget *image = gtk_image_new_from_file(	pic_buffer ); 
	gtk_container_add( GTK_CONTAINER(event_box), image );

	gtk_widget_set_events( event_box, GDK_BUTTON_PRESS_MASK );

	GSList *song_pic_filenames = NULL;
	song_pic_filenames = g_slist_append( song_pic_filenames, song_buffer);  
	song_pic_filenames = g_slist_append( song_pic_filenames, pic_buffer);	
	gtk_signal_connect( GTK_OBJECT(event_box), "button_press_event",
						G_CALLBACK(cb_from_selection_to_recording), 
						song_pic_filenames );

	gtk_fixed_put( GTK_FIXED(fixed), frame, xcoord, ycoord );

	// calculate button's position (it's relative to frame's position) 
	xcoord += button_position_x;
	ycoord += button_position_y;
	gtk_fixed_put( GTK_FIXED(fixed), event_box, xcoord, ycoord );
	song_pic_filenames = NULL;				
	#if DEBUG
		g_print( "Added a button to the screen\n" );
	#endif
}	

GSList *scan_and_sort_directory( gchar *directory_name )
{
	GError *error = NULL;
	guint flags = 0;

	GDir *directory = g_dir_open( directory_name, flags, &error );
		
	if( directory == NULL ) {
		g_warning("Couldn't not open the directory %s\n", directory_name);
		g_warning("(glib) Error: %s\n", error->message );

		g_error_free( error );

		return NULL;
	}
	g_assert( directory != NULL );

	GSList *list = NULL;

	while( TRUE ) {
		const gchar *temp = NULL;
		/* the return value of g_dir_read_name is own by glib
			Do not modify or free 
		*/
		temp = g_dir_read_name( directory );
		if( temp == NULL ) {
			// all the files scanned from the directory
			break;
		}
		#if 1
		gchar *copy = g_strndup( temp, strlen(temp) );
		// turns it into lower case
		gchar *low = g_ascii_strdown( copy, strlen(copy) );
		g_free(copy);
			
		g_print("\t\t\t %s\n", low );

		list = g_slist_append( list, (gchar *) low );
		#else
		list = g_slist_append( list, (gchar *) temp );
		#endif
	}

	list = g_slist_sort( list, string_compare );
	g_dir_close( directory );			
	
	#if 0
	{   /* prints the contents of the list */
		guint length = 0;
		gint index = 0;

		length = g_slist_length( list );
		for( index = 0; index < length; ++index ) {
			gchar *element = (gchar *) g_slist_nth_data( list, index );
			printf("%uth is %s\n", index, element );
		}
	}
	#endif

	return list;
}
/**
compares 2 filenames (skips ending + ".")
**/
gint string_compare( gconstpointer a, gconstpointer b )
{
	g_assert( a != NULL && b != NULL );	

	guint length = -1;
	gint rv = -1;

	length =  strlen( (gchar *) a );
	rv = memcmp( a, b, length - 4 );

	return rv;
}

#endif
