#ifndef _UI_C_
#define _UI_C_

#include "ui.h"
#include "callbacks.h"
#include "songselection.h"
#include "recording.h"

//#include "email.h"
//---------------------
#include <gdk/gdk.h>
//---------------------

#include "../video.h"
#include "../functionality.h"

#include <glib.h> /* Double-ended queue */
#include <string.h> /* for strlen() and pals */
#include <tny-maemo-platform-factory.h>
#include <tny-platform-factory.h>
#include <tny-msg-view.h>
#include <tny-msg-window.h>
#include <tny-msg.h>
#include <tny-account-store.h>
#include <tny-account.h>
#include <tny-camel-transport-account.h>

#define HANDLE "/usr/share/mobikid/pics/button.png"

/* Backgrounds */
#define SPLASHSCREEN "/usr/share/mobikid/pics/puzzle0.png"
#define SONG_SELECTION "/usr/share/mobikid/pics/puzzle1.png"
#define LISTENING "/usr/share/mobikid/pics/puzzle3.png"
#define SENDING "/usr/share/mobikid/pics/puzzle4.png"

/* Icons */
#define NOTE "/usr/share/mobikid/pics/puzzle3_playing.gif"
#define BIG_FRAME "/usr/share/mobikid/pics/song_frame.png"

/* Directories */
//#define EMAIL_DIRECTORY "/usr/share/mobikid/email/"
#define EMAIL_DIRECTORY "/home/user/.mobikid/"
//teemu<=>user
#define IMAGES "/home/user/.mobikid/mail"
//#define IMAGES "/usr/share/mobikid/email/mail"

typedef struct {
	guint x;
	GtkWidget *widget;
} View;

GQueue *queue = NULL;
GtkWidget *layout = NULL;
GtkAdjustment *adjustment = NULL;

/* ==== Helpers ==== */
gint string_compare( gconstpointer a, gconstpointer b );
gboolean compare_names( const gchar *name1, const gchar *suffix1,
						const gchar *name2, const gchar *suffix2  );
GSList *scan_and_sort_directory( GDir *directory );

GtkWidget *create_splash_screen();
GtkWidget *create_recording();
GtkWidget *create_listening();
GtkWidget *create_sending();

void show_splash_screen_video();

HildonWindow *ui_initialize()
{
	HildonWindow *window = NULL;
	window = HILDON_WINDOW( hildon_window_new() );

	g_signal_connect( G_OBJECT(window), "delete_event",
					  G_CALLBACK(cb_quit), NULL );

	// For keyboard events
	g_signal_connect( G_OBJECT(window), "key_press_event",
					  G_CALLBACK(cb_hardkey_press), NULL );

	/* This will capture all the events, possible even
	   screen update events which would not be a good thing.
	   button_press_event might be the best choice.
		http://library.gnome.org/devel/gtk/stable/GtkWidget.html#id4292632
	*/
	// for mouse events
	g_signal_connect( G_OBJECT(window), "button_press_event",
					  G_CALLBACK(cb_mousebutton_press), NULL );

	gtk_window_fullscreen( GTK_WINDOW(window) );

	adjustment = (GtkAdjustment *)gtk_adjustment_new( 0.0, 0.0,
							   50.0 * WINDOW_WIDTH,
							   32.0,
							   (gdouble) WINDOW_WIDTH,
							   (gdouble) WINDOW_WIDTH );
	layout = gtk_layout_new( adjustment, NULL );

	/* FIXME WINDOW_WIDTH * 50 is a quick hack to get proper sliding work */
	/* same above with the adjustment FIXME */
	gtk_layout_set_size( GTK_LAYOUT(layout), WINDOW_WIDTH * 50, WINDOW_HEIGHT );

	/*
		There are some trade-offs concerning the gtk_fixed but
		none of them are a cornern for Mobikid
		(For the trade-offs check out the documentation of gtk_fixed)

	*/
	queue = g_queue_new();
	View *temp = NULL;

	/* Splash screen */
	temp = g_try_malloc0( sizeof(View) );
	g_assert( temp );

	temp->widget = create_splash_screen();
	temp->x = WINDOW_WIDTH * 0;

	gtk_layout_put( GTK_LAYOUT(layout), temp->widget, temp->x, 0 );
	g_queue_push_tail( queue, temp );

	/* song selection, may return NULL */
	temp = g_try_malloc0( sizeof(View) );
	g_assert( temp );

	temp->widget = songselection_create();
	if( temp->widget == NULL ) {
		g_warning("Can't create song selection screen\n");

		return NULL;
	}
	temp->x = WINDOW_WIDTH * 1;

	gtk_layout_put( GTK_LAYOUT(layout), temp->widget, temp->x, 0 );
	g_queue_push_tail( queue, temp );

	/* recording */
	temp = g_try_malloc0( sizeof(View) );
	g_assert( temp );

	temp->widget = recording_create();
	temp->x = WINDOW_WIDTH * 2;

	gtk_layout_put( GTK_LAYOUT(layout), temp->widget, temp->x, 0 );
	g_queue_push_tail( queue, temp );

	/* listening */
	temp = g_try_malloc0( sizeof(View) );
	g_assert( temp );

	temp->widget = create_listening();
	temp->x = WINDOW_WIDTH * 3;

	gtk_layout_put( GTK_LAYOUT(layout), temp->widget, temp->x, 0 );
	g_queue_push_tail( queue, temp );

	/* sending */
	temp = g_try_malloc0( sizeof(View) );
	g_assert( temp );

	temp->widget = create_sending();
	temp->x = WINDOW_WIDTH * 4;

	gtk_layout_put( GTK_LAYOUT(layout), temp->widget, temp->x, 0 );
	g_queue_push_tail( queue, temp );

	/* add to layout */
	gtk_container_add( GTK_CONTAINER(window), layout );

	return window;
}

void ui_deinitialize()
{
	/*
	Free all the views, filenames and other leaky stuff here
	*/
	songselection_destroy();
 	recording_destroy();
 	while( g_queue_get_length (queue) > 0 )
		g_free( g_queue_pop_head( queue ));
	g_queue_free( queue );
  	queue = NULL;
	g_free( adjustment );
	adjustment = NULL;
	layout = NULL;

}


void ui_show( HildonWindow *window )
{
	g_assert( window );

	show_splash_screen_video();
	gtk_widget_show_all( GTK_WIDGET(window) );
}

void ui_slide( Direction dir )
{
	g_assert( adjustment != NULL );

#if 1
	#if DEBUG
	g_print("ui_slide(): The new implementation\n");
	#endif

	const guint iterations = 10;
	gdouble value = gtk_adjustment_get_value( GTK_ADJUSTMENT(adjustment) );
	gint cnt = 0;
	gint increment = WINDOW_WIDTH/iterations;

	View *first = NULL, *last = NULL;
	if( dir == RIGHT ) {
		#if DEBUG
		g_print("\r Right reorder\n");
		#endif

		/* No need to set the direction because it
			right by default */
		first = (View *) g_queue_pop_head( queue );
		if( first == NULL ) {
			g_warning("ui_slide(): first. Queue is empty");
		}

		last = (View *) g_queue_peek_tail( queue );
		guint tempx = last->x + WINDOW_WIDTH;
		first->x = tempx;

		g_queue_push_tail( queue, first );

		gtk_layout_move( GTK_LAYOUT(layout), first->widget,
								 first->x,
									 0 );
	} else if( dir == LEFT ) {
		#if DEBUG
		g_print("\r Left reorder\n");
		#endif

		/* Set movement's direction for left */
		increment = -increment;

		last = (View *) g_queue_pop_tail( queue );
		if( last == NULL ) {
			g_warning("ui_slide(): last. Queue is empty");
		}

		first = (View *) g_queue_peek_head( queue );
		guint tempx = first->x - WINDOW_WIDTH;
		last->x = tempx;

		g_queue_push_head( queue, last );

		gtk_layout_move( GTK_LAYOUT(layout), last->widget,
							 last->x,
								 0 );

		/* fixes the bug which shows the screen as white
			after a slide to the left
		*/
		gtk_main_iteration();
	} else {
		#if DEBUG
		g_warning("slide_new received errornous parameter\n");
		#endif
	}

	for( cnt = 0; cnt < iterations; ++cnt ) {
		value += increment;

		/*
		if( dir == LEFT ) {
			gtk_main_iteration();
		}
		*/
		gtk_main_iteration();

		gtk_adjustment_set_value( GTK_ADJUSTMENT(adjustment), value );
	}

	/* It's true only for the very first slide
	   The first viewed view is assumed to be
	   the splash screen */
	static gboolean after_splash_screen = TRUE;
	if( after_splash_screen == TRUE ) {

		/* remove the splash screen */
		/* if screen was slided to the right it's the last one */
		if( dir == RIGHT ) {
			g_queue_pop_tail( queue );
		}
		else { /* otherwise the first one */
			g_queue_pop_head( queue );
		}

		after_splash_screen = FALSE;
	}

#else
	/* Old implementation without splash removal and
	   "circle effect"
	*/
	#if DEBUG
	g_print("ui_slide(): The original implementation\n");
	#endif
	g_assert( adjustment != NULL );

	const guint iterations = 10;
	gdouble value = gtk_adjustment_get_value( GTK_ADJUSTMENT(adjustment) );
	gint cnt = 0;
	gint increment = WINDOW_WIDTH/iterations;

	if( dir == LEFT ) {
		increment = -increment;
	}

	for( cnt = 0; cnt < iterations; ++cnt ) {
		value += increment;

		gtk_adjustment_set_value( GTK_ADJUSTMENT(adjustment), value );
		gtk_main_iteration();
	}
#endif

	#if DEBUG
	g_print("The screen slides\n");
	#endif
}

/**
	the intro video is be showed after all the screens are created and splash screen is displayed
**/
void show_splash_screen_video()
{
	g_assert( queue != NULL );

	View *first = NULL;

	first = (View *) g_queue_peek_head( queue );
	if( video_determine_file() == TRUE ) {
		GtkWidget *video_screen = gtk_drawing_area_new();
		gtk_drawing_area_size( GTK_DRAWING_AREA(video_screen), 650, 440 );
	  	gtk_fixed_put( GTK_FIXED(first->widget), video_screen, 75, 10 );

	/*-----------------------------------------------------------------------------*/
	//gtk_widget_add_events (video_screen,GDK_BUTTON_PRESS_MASK);
	gtk_widget_set_events( video_screen, GDK_BUTTON_PRESS_MASK );
	g_signal_connect( G_OBJECT(video_screen), "button-press-event",
					  G_CALLBACK(cb_from_splash_screen_to_selection),
					  (gpointer) RIGHT );
	/*-----------------------------------------------------------------------------*/
	video_playback_start( video_screen );
	}
	else
	{
		ui_slide( RIGHT );
	}
}

GtkWidget *create_splash_screen()
{
	GtkWidget *fixed = gtk_fixed_new();

	#if DEBUG
	g_print( "create splash screen\n");
	GtkWidget *button = gtk_button_new();
	GtkWidget *right_handle = NULL;

	right_handle = gtk_image_new_from_file( HANDLE );
	gtk_button_set_image(GTK_BUTTON(button), right_handle);
	g_signal_connect( G_OBJECT(button), "clicked",
					  G_CALLBACK(cb_from_splash_screen_to_selection),
					  (gpointer) RIGHT );
						gtk_fixed_put( GTK_FIXED(fixed), button,
						WINDOW_WIDTH - 130, 165 );
	#endif

	GtkWidget *background = NULL;
	background = gtk_image_new_from_file( SPLASHSCREEN );
	gtk_fixed_put( GTK_FIXED(fixed), background, 0, 0 );


	return fixed;
}



GtkWidget *create_listening()
{
	GtkWidget *fixed = NULL;
	GtkWidget *background = NULL;
	GtkWidget *handle = NULL;
	GtkWidget *note = NULL;

	fixed = gtk_fixed_new();
	handle = gtk_image_new_from_file( HANDLE );

	/* left button */
	/* for some unknown reason the left button doesn't
		without a second pointer to the handle */
	GtkWidget *handle2 = gtk_image_new_from_file( HANDLE );

	GtkWidget *button = gtk_button_new();
	gtk_button_set_image(GTK_BUTTON(button), handle2);
	g_signal_connect( G_OBJECT(button), "clicked",
					  G_CALLBACK(cb_from_listening_to_recording),
					  (gpointer) LEFT );

	gtk_fixed_put( GTK_FIXED(fixed), button,
					5,
					165 );

	// right button
	{
	GtkWidget *button = gtk_button_new();
	gtk_button_set_image(GTK_BUTTON(button), handle);
	g_signal_connect( G_OBJECT(button), "clicked",
					  G_CALLBACK(cb_from_listening_to_sending),
					  (gpointer) RIGHT );

	gtk_fixed_put( GTK_FIXED(fixed), button,
					WINDOW_WIDTH - 135,
					165 );
	}

	// background
	background = gtk_image_new_from_file( LISTENING );
	gtk_fixed_put( GTK_FIXED(fixed), background, 0, 0 );

	/* Adds note */
	note = gtk_image_new_from_file( NOTE );
	gtk_fixed_put( GTK_FIXED(fixed), note, 345, 90 );

	return fixed;
}

GtkWidget *create_sending()
{
	GtkWidget *fixed = NULL;
	GtkWidget *background = NULL;
	GtkWidget *handle = NULL;
	gsize length;
	//gchar* filename2;
	//gchar* fntemp;

	GKeyFile* address;
	address=g_key_file_new();
	g_key_file_load_from_file (address,IMAGES,G_KEY_FILE_NONE,NULL);

	fixed = gtk_fixed_new();
	handle = gtk_image_new_from_file( HANDLE );

	#if DEBUG
	// left button (not really needed, but useful for debugging)
	GtkWidget *button = gtk_button_new();
	gtk_button_set_image(GTK_BUTTON(button), handle);
	g_signal_connect( G_OBJECT(button), "clicked",
					  G_CALLBACK(cb_slide), (gpointer) LEFT );
	gtk_fixed_put( GTK_FIXED(fixed), button, 5,	165 );
	#endif

	// right button
	{
	GtkWidget *button = gtk_button_new();
	gtk_button_set_image(GTK_BUTTON(button), handle);
	g_signal_connect( G_OBJECT(button), "clicked",
					  G_CALLBACK(cb_from_sending_to_selection),
					  (gpointer) RIGHT );

	gtk_fixed_put( GTK_FIXED(fixed), button,
				   WINDOW_WIDTH - 135,
				   165 );
	}


	/* Functionality for the right button will added later */
	background = gtk_image_new_from_file( SENDING );
	gtk_fixed_put( GTK_FIXED(fixed), background, 0, 0 );

	/* For consistency these should be the same as the
		paddings used in song selection screen.
		Same goes for the other values too!
	 */
	// padding between the left/top "border" and the buttons
	const guint padding_left = 124, padding_top = 48; // 28??
	// padding between the buttons (in pixels)
	const guint spacing_x = 24, spacing_y = 32;
	// size of song frame
	const guint frame_width = 120, frame_heigth = 120;
	// buttons position (relative to frame's position)
	const guint button_position_x = 9, button_position_y = 9;

	const guint max_rows = 3, max_columns = 4;

	/* the receivers of the emails */
	g_print("Email start\n");

	GSList *addresses = NULL;
	GSList *kuvat=NULL;
	gchar *ostmp;
	gchar *kuvtmp;
	gchar *no;
	/* Error's aren't handled */
	//GDir *directory = g_dir_open( EMAIL_DIRECTORY, 0 , NULL );
	//const gchar *tmp = NULL;
	int i=1;

	while(i<50)
	{
	no=g_strdup_printf("%d",i);

	if(g_key_file_has_group(address,no)==TRUE)
	{
	kuvtmp=g_key_file_get_string(address,no,"kuva",NULL);
	ostmp=g_key_file_get_string(address,no,"mail",NULL);
	addresses = g_slist_append( addresses, ostmp );
	kuvat=g_slist_append( kuvat, kuvtmp );
	i++;
	}
		else
		{
		i++;
		//bool_loytyi=FALSE;
		}
	}

	//g_dir_close( directory );
	g_print("Names are read, now print!\n");

	guint current_column = 0;
	guint current_row = 0;

	guint index = 0;
	for( index = 0; index < g_slist_length(addresses); ++index ) {
		gchar *filename = g_slist_nth_data( kuvat, index );
		gchar* osoite=g_slist_nth_data( addresses, index );
		//fntemp=strdup(filename);
		g_print("\t%i %s\n", index, filename );

		if( current_column >= max_columns ) {
			current_column = 0;
			current_row++;
			if( current_row >= max_rows ) {
				#if DEBUG
				g_print("out of screen space\n");
				#endif
				break;
			}
		}

		// calculate frame's position
		guint xcoord = 0;
		guint ycoord = 0;

		xcoord += (current_column * frame_width) + padding_left;
		ycoord += (current_row * frame_heigth) + padding_top;

		xcoord += spacing_x * current_column;
		ycoord += spacing_y * current_row;

		current_column++;

		/* the frame first */
		GtkWidget *frame = gtk_image_new_from_file( BIG_FRAME );
		gtk_fixed_put( GTK_FIXED(fixed), frame,	xcoord, ycoord );

		/* then the picture */
		GtkWidget *event_box = gtk_event_box_new();
		// This string ought to be freed
		GString *path_string = g_string_new( EMAIL_DIRECTORY );
		g_string_append( path_string, filename );

		//filename2=g_strconcat (fntemp,".gif",NULL);
		//g_string_append(filename2,".gif");

		//g_string_append( path_string, ".gif" );
		//g_string_append( path_string, ".jpg" );
		gchar *path = g_string_free( path_string, FALSE );

		GtkWidget *image = gtk_image_new_from_file(	path );

		gtk_container_add( GTK_CONTAINER(event_box), image );
		gtk_widget_set_events( event_box, GDK_BUTTON_PRESS_MASK );
		printf("\nSignal filename2: %s\n",osoite);
		gtk_signal_connect( GTK_OBJECT(event_box), "button_press_event",
							G_CALLBACK(cb_send_email),osoite);/*email_send*/

		/* calculate button's position (it's relative to frame's position) */
		xcoord += button_position_x;
		ycoord += button_position_y;

		gtk_fixed_put( GTK_FIXED(fixed), event_box, xcoord, ycoord );
	}

	g_print("Email end\n");

	//g_free(fntemp);
	//g_free(filename2);
	return fixed;
}

#endif
